from datetime import datetime
from typing import Dict, List

from nisystemlink.clients.core._uplink._json_model import JsonModel
from pydantic import model_validator

from ._alarm import AlarmSeverityLevel, AlarmTransitionType


class CreateAlarmTransition(JsonModel):
    """Contains information about a transition used to create or update an instance of an alarm."""

    transition_type: AlarmTransitionType
    """Specifies a type of alarm transition: CLEAR or SET."""

    occurred_at: datetime | None = None
    """The date and time when the transition occurred."""

    severity_level: int | None = 2
    """The severity of the transition.

    Valid values for CLEAR transitions are [-1, -1].
    Valid values for SET transitions are [1, 2147483647].
    Note that the SystemLink Alarm UI only has display strings for SET severities in the range [1, 4].
    The AlarmSeverityLevel enum provides values for standard severity levels.
    """

    value: str | None = None
    """The value that caused the alarm to transition."""

    condition: str | None = None
    """A description of the condition associated with the transition."""

    short_text: str | None = None
    """A short description of the condition."""

    detail_text: str | None = None
    """A detailed description of the condition."""

    keywords: List[str] | None = None
    """Words or phrases associated with a transition.

    Useful for attaching metadata to a transition which could aid in an investigation
    into an alarm's root cause in the future.
    """

    properties: Dict[str, str] | None = None
    """Key-value pair metadata associated with a transition.

    Useful for attaching additional metadata to a transition which could aid in an investigation
    into an alarm's root cause in the future. Property keys must be between 1 and 255 characters.
    Property values can be up to 1000 characters.
    """


class SetAlarmTransition(CreateAlarmTransition):
    """Contains information about a SET transition used to create or update an instance of an alarm.

    This is a convenience class that automatically sets transition_type to SET.
    """

    transition_type: AlarmTransitionType = AlarmTransitionType.SET

    @model_validator(mode="after")
    def _set_transition_type(self) -> "SetAlarmTransition":
        self.transition_type = AlarmTransitionType.SET
        return self


class ClearAlarmTransition(CreateAlarmTransition):
    """Contains information about a CLEAR transition used to clear an instance of an alarm.

    This is a convenience class that automatically sets transition_type to CLEAR
    and severity_level to -1 (CLEAR severity).
    """

    transition_type: AlarmTransitionType = AlarmTransitionType.CLEAR
    severity_level: int | None = AlarmSeverityLevel.CLEAR

    @model_validator(mode="after")
    def _set_clear_defaults(self) -> "ClearAlarmTransition":
        self.transition_type = AlarmTransitionType.CLEAR
        self.severity_level = AlarmSeverityLevel.CLEAR
        return self


class CreateOrUpdateAlarmRequest(JsonModel):
    """Contains information about the alarm to create or update.

    If an alarm is being updated, only alarmId, workspace, and transition are applied.
    """

    alarm_id: str
    """A value meant to uniquely identify the particular process or condition tracked by the alarm.

    For example, alarms created by a tag rule engine might have their alarmIds set to the
    concatenation of the path of the tag which caused the rule to be triggered and the ID of the rule.
    """

    workspace: str | None = None
    """The ID of the workspace in which to create or update the alarm.

    When not specified, the default workspace is used based on the requesting user.
    """

    transition: CreateAlarmTransition
    """Contains information about a transition used to create or update an instance of an alarm.

    Consider using SetAlarmTransition to trigger an alarm or ClearAlarmTransition to clear an alarm.
    These convenience classes automatically set the appropriate transition type and severity level.
    """

    notification_strategy_ids: List[str] | None = None
    """The IDs of the notification strategies which should be triggered.

    These are triggered if this request results in an alarm being created or
    transitioning to a new highest severity.
    """

    created_by: str | None = None
    """An identifier for who or what created the alarm.

    This is usually used to identify the particular rule engine which requested
    that the alarm be created.
    """

    channel: str | None = None
    """An identifier for the tag or resource associated with the alarm."""

    resource_type: str | None = None
    """The type of resource associated with the alarm."""

    display_name: str | None = None
    """Optional display name for the alarm. Display names do not need to be unique."""

    description: str | None = None
    """Optional description text for the alarm."""

    keywords: List[str] | None = None
    """Words or phrases associated with the alarm.

    Alarms can be tagged with keywords to make it easier to find them with queries.
    """

    properties: Dict[str, str] | None = None
    """The alarm's custom properties. Property keys must be between 1 and 255 characters.
    Property values can be up to 1000 characters."""



"""Implementation of Alarm Client"""

from typing import List, Literal, overload

from nisystemlink.clients import core
from nisystemlink.clients.core._uplink._base_client import BaseClient
from nisystemlink.clients.core._uplink._methods import delete, get, post
from uplink import Field, Path, retry

from . import models


@retry(
    when=retry.when.status(408, 429, 502, 503, 504),
    stop=retry.stop.after_attempt(5),
    on_exception=retry.CONNECTION_ERROR,
)
class AlarmClient(BaseClient):

    def __init__(self, configuration: core.HttpConfiguration | None = None):
        """Initialize an instance.

        Args:
            configuration: Defines the web server to connect to and information about
                how to connect. If not provided, the
                :class:`HttpConfigurationManager <nisystemlink.clients.core.HttpConfigurationManager>`
                is used to obtain the configuration.

        Raises:
            ApiException: if unable to communicate with the `/nialarm` Service.
        """
        if configuration is None:
            configuration = core.HttpConfigurationManager.get_configuration()

        super().__init__(configuration, base_path="/nialarm/v1/")

    @post(
        "acknowledge-instances-by-instance-id",
        args=[Field("instanceIds"), Field("forceClear")],
    )
    def acknowledge_alarms(
        self, instance_ids: List[str], *, force_clear: bool = False
    ) -> models.AcknowledgeAlarmsResponse:
        """Acknowledges one or more alarm instances by their instance IDs.

        Args:
            instance_ids: List of instance IDs (unique occurrence identifiers) of the alarms to acknowledge.
                 These are the server-generated IDs returned when creating/updating alarms,
                 not the user-defined alarm_id.
            force_clear: Whether or not the affected alarms should have their clear field set to true.
                         Defaults to False.

        Returns:
            A response containing the instance IDs that were successfully acknowledged,
            the instance IDs that failed, and error details for failures.

        Raises:
            ApiException: if unable to communicate with the `/nialarm` Service or provided invalid arguments.
        """
        ...

    @overload
    def create_or_update_alarm(  # noqa: E704
        self,
        request: models.CreateOrUpdateAlarmRequest,
        *,
        ignore_conflict: Literal[False] = False,
    ) -> str: ...

    @overload
    def create_or_update_alarm(  # noqa: E704
        self,
        request: models.CreateOrUpdateAlarmRequest,
        *,
        ignore_conflict: Literal[True],
    ) -> str | None: ...

    def create_or_update_alarm(
        self,
        request: models.CreateOrUpdateAlarmRequest,
        *,
        ignore_conflict: bool = False,
    ) -> str | None:
        """Creates or updates an instance, or occurrence, of an alarm.

        Creates or updates an alarm based on the requested transition and the state
        of the current active alarm with the given alarm_id (specified in the request).
        Multiple calls with the same alarm_id will update the same alarm instance.

        Args:
            request: The request containing alarm_id (user-defined identifier),
                    transition details, and other alarm properties.
            ignore_conflict: If True, 409 Conflict errors will be ignored and None will be returned.
                           If False (default), 409 errors will raise an ApiException.
                           Setting this to True is useful for stateless applications that want to
                           attempt state transitions without checking the current alarm state first.

        Returns:
            The instance_id (unique occurrence identifier) of the created or modified alarm.
            Use this ID for operations like get_alarm(), delete_alarm(), or acknowledge.
            Returns None if ignore_conflict is True and a 409 Conflict occurs.

        Raises:
            ApiException: if unable to communicate with the `/nialarm` Service or provided invalid arguments.
                A 409 Conflict error occurs when the request does not represent a valid transition
                for an existing alarm, such as attempting to clear an alarm which is already clear,
                or attempting to set an alarm which is already set at the given severity level.
                This error can be suppressed by setting ignore_conflict=True.
        """
        try:
            return self._create_or_update_alarm(request)
        except core.ApiException as e:
            if ignore_conflict and e.http_status_code == 409:
                return None
            raise

    @post("instances", return_key="instanceId")
    def _create_or_update_alarm(
        self, request: models.CreateOrUpdateAlarmRequest
    ) -> str:
        """Internal implementation of create_or_update_alarm."""
        ...

    @get("instances/{instance_id}", args=[Path("instance_id")])
    def get_alarm(self, instance_id: str) -> models.Alarm:
        """Gets an alarm by its instance_id.

        Args:
            instance_id: The unique instance ID (occurrence identifier) of the alarm to retrieve.
                This is the server-generated ID returned from create_or_update_alarm(),
                not the user-defined alarm_id.

        Returns:
            The alarm with the specified instance_id.

        Raises:
            ApiException: if unable to communicate with the `/nialarm` Service or provided invalid arguments.
        """
        ...

    @delete("instances/{instance_id}", args=[Path("instance_id")])
    def delete_alarm(self, instance_id: str) -> None:
        """Deletes an alarm by its instance_id.

        Args:
            instance_id: The unique instance ID (occurrence identifier) of the alarm to delete.
                This is the server-generated ID returned from create_or_update_alarm(),
                not the user-defined alarm_id.

        Raises:
            ApiException: if unable to communicate with the `/nialarm` Service or provided invalid arguments.
        """
        ...

    @post("delete-instances-by-instance-id", args=[Field("instanceIds")])
    def delete_alarms(self, instance_ids: List[str]) -> models.DeleteAlarmsResponse:
        """Deletes multiple alarm instances by their instance IDs.

        Args:
            instance_ids: List of instance IDs (unique occurrence identifiers) of the alarms to delete.
                 These are the server-generated IDs returned when creating/updating alarms,
                 not the user-defined alarm_id.

        Returns:
            A response containing lists of successfully deleted and failed instance IDs,
            along with error information for failures.

        Raises:
            ApiException: if unable to communicate with the `/nialarm` Service or provided invalid arguments.
        """
        ...

    @post("query-instances-with-filter")
    def query_alarms(
        self, request: models.QueryAlarmsWithFilterRequest
    ) -> models.QueryAlarmsWithFilterResponse:
        """Queries for instances, or occurrences, of alarms using Dynamic LINQ.

        Specifying an empty JSON object in the request body will result in all alarms being returned.

        Args:
            request: The request containing filter information and query options.

        Returns:
            A response containing the list of alarms that match the query, along with
            optional total count and continuation token for pagination.

        Raises:
            ApiException: if unable to communicate with the `/nialarm` Service or provided invalid arguments.
        """
        ...



from typing import List

from nisystemlink.clients.core import ApiError
from nisystemlink.clients.core._uplink._json_model import JsonModel


class AlarmInstancesPartialSuccess(JsonModel):
    """Base class for partial success responses containing success/failure lists and error information."""

    failed: List[str]
    """The instanceIds that failed the operation."""

    error: ApiError | None = None
    """The error that occurred during the operation."""



from datetime import datetime
from typing import Dict, List

from nisystemlink.clients.core._uplink._json_model import JsonModel

from ._dashboard import Dashboard
from ._execution_definition import ExecutionDefinition


class TestPlanTemplateBase(JsonModel):
    """Contains information about a test plan template."""

    name: str | None = None
    """Name of the test plan template."""

    template_group: str | None = None
    """The template group defined by the user."""

    product_families: List[str] | None = None
    """Array of product families to which the test plan template belongs."""

    part_numbers: List[str] | None = None
    """Array of part numbers of the products to which the test plan template belongs."""

    summary: str | None = None
    """Summary of the test plan template."""

    description: str | None = None
    """Description of the test plan created from this template."""

    test_program: str | None = None
    """Test program name of the test plan created from this template."""

    estimated_duration_in_seconds: int | None = None
    """The estimated time in seconds for executing the test plan created from this template."""

    system_filter: str | None = None
    """The LINQ filter string is used to filter the potential list of
    systems capable of executing test plans created from this template.
    """

    dut_filter: str | None = None
    """The LINQ filter string is used to filter the potential list of
    DUTs capable of executing test plans created from this template.
    """

    execution_actions: List[ExecutionDefinition] | None = None
    """Defines the executions that will be used for test plan actions
    created from this template.
    """

    file_ids: List[str] | None = None
    """Array of file IDs associated with the test plan template."""

    workspace: str | None = None
    """ID of the workspace where the test plan template belongs.
    Default workspace will be taken if the value is not given.
    """

    properties: Dict[str, str] | None = None
    """Properties of the test plan created from this template as key-value pairs."""

    dashboard: Dashboard | None = None
    """Defines a dashboard reference for a test plan."""


class TestPlanTemplate(TestPlanTemplateBase):
    """Contains response information for test plan template."""

    __test__ = False

    id: str | None = None
    """The globally unique id of the test plan template."""

    name: str | None = None
    """Name of the test plan template."""

    created_by: str | None = None
    """ID of the user who created the test plan template."""

    updated_by: str | None = None
    """ID of the user who most recently updated the test plan template."""

    created_at: datetime | None = None
    """ISO-8601 formatted timestamp indicating when the
    test plan template was created."""

    updated_at: datetime | None = None
    """ISO-8601 formatted timestamp indicating when the
    test plan template was most recently updated."""



# -*- coding: utf-8 -*-

"""Implementation of PathConstants."""

import os
import pathlib
import typing

from nisystemlink.clients.core._internal._classproperty_support import (
    ClasspropertySupport,
)
from typing_extensions import final


@final
class PathConstants(metaclass=ClasspropertySupport):
    """Provides file and directory paths for the SystemLink client APIs."""

    COMPANY_NAME = "National Instruments"

    PRODUCT_NAME = "Skyline"

    _application_data_directory: pathlib.Path | None = None

    _salt_data_directory: pathlib.Path | None = None

    def __init_subclass__(cls) -> None:
        raise TypeError("type 'PathConstants' is not an acceptable base type")

    def __init__(self) -> None:
        raise TypeError("Can't instantiate static class 'PathConstants'")

    @ClasspropertySupport.classproperty
    def application_data_directory(self) -> pathlib.Path:  # noqa: D401
        """Get the platform-specific path to the common NI Application Data directory.

        Returns:
            The platform-specific path to the common NI Application Data directory.
        """
        if self._application_data_directory is None:
            if os.name == "nt":
                PathConstants._application_data_directory = (
                    self._windows_application_data_directory()
                )
            else:
                PathConstants._application_data_directory = pathlib.Path(
                    "/etc/natinst/niskyline"
                )
        return typing.cast(pathlib.Path, self._application_data_directory)

    @ClasspropertySupport.classproperty
    def salt_data_directory(self) -> pathlib.Path:  # noqa: D401
        """Get the platform-specific path to the salt minion Data directory.

        Returns:
            The platform-specific path to the salt minion Data directory.
        """
        if self._salt_data_directory is None:
            if os.name == "nt":
                PathConstants._salt_data_directory = self._windows_salt_data_directory()

        return typing.cast(pathlib.Path, self._salt_data_directory)

    @classmethod
    def _windows_programdata_directory(cls) -> pathlib.Path:
        """Get the path of Windows PROGRAMDATA directory.

        Raises:
            RuntimeError: if this method is called on a non-Windows system.
            RuntimeError: if the ProgramData folder cannot be found.

        Returns:
            pathlib.Path: Windows PROGRAMDATA directory path.
        """
        if os.name != "nt":
            raise RuntimeError("This function is for Windows only")

        programdata_dir = None

        try:
            from . import _winpaths

            programdata_dir = pathlib.Path(
                _winpaths.get_path(_winpaths.FOLDERID.ProgramData)
            )
        except Exception:
            env_programdata_dir = os.getenv("PROGRAMDATA")

            if env_programdata_dir:
                programdata_dir = pathlib.Path(env_programdata_dir)

        if programdata_dir is not None and programdata_dir.exists():
            return programdata_dir

        # Last fallback option, assume that it is in C:/
        programdata_dir = pathlib.Path("C:/ProgramData")

        if not programdata_dir.exists():
            raise RuntimeError("Cannot find ProgramData folder")

        return programdata_dir

    @classmethod
    def _windows_application_data_directory(cls) -> pathlib.Path:
        """Get the NI Application Data directory on Windows.

        Returns:
            pathlib.Path: The NI Application Data directory on Windows.

        Raises:
            RuntimeError: if this method is called on a non-Windows system.
            RuntimeError: if the ProgramData folder cannot be found.
        """
        programdata_dir = cls._windows_programdata_directory()

        return programdata_dir / cls.COMPANY_NAME / cls.PRODUCT_NAME

    @classmethod
    def _windows_salt_data_directory(cls) -> pathlib.Path:
        """Get the salt minion Data directory on Windows.

        Returns:
            pathlib.Path: The salt minion Data directory on Windows.

        Raises:
            RuntimeError: if this method is called on a non-Windows system.
            RuntimeError: if the ProgramData folder cannot be found.
        """
        programdata_dir = cls._windows_programdata_directory()
        return programdata_dir / cls.COMPANY_NAME / "salt"



from datetime import datetime

from nisystemlink.clients.core._uplink._json_model import JsonModel


class AssetUtilizationHistoryItem(JsonModel):
    """Model representing an asset utilization history record."""

    utilization_identifier: str | None = None
    """String representing the unique identifier of an asset utilization history record."""

    asset_identifier: str | None = None
    """String representing the unique identifier of an asset."""

    minion_id: str | None = None
    """Identifier of the minion where the asset is located."""

    category: str | None = None
    """String representing the utilization task category."""

    task_name: str | None = None
    """String representing the name of the task."""

    user_name: str | None = None
    """String representing the name of the operator who utilized the asset."""

    start_timestamp: datetime | None = None
    """A date time value which can be used to specify the start of an utilization."""

    end_timestamp: datetime | None = None
    """A date time value which can be used to specify the end of an utilization."""

    heartbeat_timestamp: datetime | None = None
    """A date time value which can be used to specify the heartbeat of an utilization."""



from typing import List

from nisystemlink.clients.core._uplink._with_paging import WithPaging

from ._notebook_metadata import NotebookMetadata


class PagedNotebooks(WithPaging):
    """Model for a query notebooks response."""

    notebooks: List[NotebookMetadata] | None = None
    """The list of notebooks."""



from ._file_metadata import FileMetadata
from ._file_query_order_by import FileQueryOrderBy, FileLinqQueryOrderBy
from ._file_query_response import FileQueryResponse
from ._link import Link
from ._operations import V1Operations
from ._update_metadata import UpdateMetadataRequest
from ._file_linq_query import FileLinqQueryRequest, FileLinqQueryResponse

# flake8: noqa



from ._acknowledge_alarms_response import AcknowledgeAlarmsResponse
from ._alarm import Alarm, AlarmSeverityLevel, AlarmTransition, AlarmTransitionType
from ._create_or_update_alarm_request import (
    ClearAlarmTransition,
    CreateAlarmTransition,
    CreateOrUpdateAlarmRequest,
    SetAlarmTransition,
)
from ._delete_alarms_response import DeleteAlarmsResponse
from ._query_alarms_request import (
    AlarmOrderBy,
    QueryAlarmsWithFilterRequest,
    TransitionInclusionOption,
)
from ._query_alarms_response import QueryAlarmsWithFilterResponse

# flake8: noqa



from typing import Dict, List

from nisystemlink.clients.core._uplink._json_model import JsonModel


class ColumnMetadataPatch(JsonModel):
    """Specifies column properties to add, modify, or delete when editing table metadata."""

    name: str
    """The name of the column to modify."""

    properties: Dict[str, str | None]
    """The properties to modify. A map of key value properties containing the metadata
    to be added or modified. Set a property value to ``None`` to delete the property."""


class ModifyTableRequest(JsonModel):
    """Contains the metadata properties to modify. Values not included will remain unchanged."""

    metadata_revision: int | None = None
    """When specified, this is an integer that must match the last known
    revision number of the table, incremented by one. If it doesn't match the
    current ``metadataRevision`` incremented by one at the time of execution, the
    modify request will be rejected with a 409 Conflict. This is used to ensure
    that changes to this table's metadata are based on a known, previous
    state."""

    name: str | None = None
    """The new name of the table. Set to ``None`` to reset the name to the table's ID."""

    test_result_id: str | None = None
    """The new test result ID associated with the table. Set to ``None`` or an empty string to
    remove the test result ID. Added in version 2 of the
    :py:attr:`nisystemlink.clients.dataframe.models.OperationsV1.modify_metadata` operation. Older
    versions of the service will ignore this value."""

    workspace: str | None = None
    """The new workspace for the table. Set to ``None`` to reset to the
    default workspace. Changing the workspace requires permission to delete the
    table in its current workspace and permission to create the table in its new
    workspace."""

    properties: Dict[str, str | None] | None = None
    """The properties to modify. A map of key value properties containing the
    metadata to be added or modified. Set a property value to ``None`` to
    delete the property."""

    columns: List[ColumnMetadataPatch] | None = None
    """Updates to the column properties. Cannot add or remove columns, or change the name of a column."""



from ._query_assets_response import QueryAssetsResponse
from ._asset import Asset
from ._create_asset_request import CreateAssetRequest
from ._create_assets_partial_success_response import CreateAssetsPartialSuccessResponse
from ._delete_assets_response import DeleteAssetsResponse
from ._query_assets_request import AssetField, QueryAssetsRequest, _QueryAssetsRequest
from ._query_assets_response import QueryAssetsResponse
from ._asset_location import (
    AssetLocation,
    AssetLocationForCreate,
    AssetPresence,
    AssetPresenceWithSystemConnection,
    AssetPresenceStatus,
)
from ._asset_calibration import (
    CalibrationMode,
    CalibrationStatus,
    SelfCalibration,
    TemperatureSensor,
    ExternalCalibration,
)
from ._asset_types import AssetBusType, AssetDiscoveryType, AssetType
from ._link_files_partial_success_response import LinkFilesPartialSuccessResponse
from ._query_asset_utilization_history_request import (
    QueryAssetUtilizationHistoryRequest,
    UtilizationOrderBy,
)
from ._asset_utilization_history_item import AssetUtilizationHistoryItem
from ._asset_utilization_history_response import AssetUtilizationHistoryResponse
from ._asset_identification import AssetIdentification
from ._start_utilization_request import StartUtilizationRequest
from ._start_utilization_partial_success_response import (
    StartUtilizationPartialSuccessResponse,
)
from ._update_utilization_partial_success_response import (
    UpdateUtilizationPartialSuccessResponse,
)

# flake8: noqa



"""Model for start utilization request."""

from datetime import datetime
from typing import List

from nisystemlink.clients.core._uplink._json_model import JsonModel

from ._asset_identification import AssetIdentification


class StartUtilizationRequest(JsonModel):
    """Request model for starting asset utilization tracking."""

    utilization_identifier: str | None = None
    """String representing the unique identifier of an asset utilization history record."""

    minion_id: str | None = None
    """Identifier of the minion where the utilized assets are located."""

    asset_identifications: List[AssetIdentification] | None = None
    """Array of the identification information for the assets which are utilized.
    The maximum number of asset identifications allowed per request is 100."""

    utilization_category: str | None = None
    """String representing the utilization category."""

    task_name: str | None = None
    """String representing the name of the task."""

    user_name: str | None = None
    """String representing the name of the operator who utilized the asset."""

    utilization_timestamp: datetime | None = None
    """A date time value which can be used to specify the start of an utilization."""



from datetime import datetime
from typing import Dict, List

from nisystemlink.clients.core._uplink._json_model import JsonModel
from nisystemlink.clients.testmonitor.models._named_value import NamedValue
from nisystemlink.clients.testmonitor.models._step import Status
from nisystemlink.clients.testmonitor.models._step_data import StepData


class BaseStepRequest(JsonModel):
    step_id: str | None
    """Step ID."""

    result_id: str
    """Result ID."""

    parent_id: str | None = None
    """Parent step ID."""

    data: StepData | None = None
    """Data returned by the test step."""

    data_model: str | None = None
    """Data model for the step."""

    started_at: datetime | None = None
    """ISO-8601 formatted timestamp indicating when the test result began."""

    status: Status | None = None
    """The status of the step."""

    step_type: str | None = None
    """Step type."""

    total_time_in_seconds: float | None = None
    """Total number of seconds the step took to execute."""

    inputs: List[NamedValue] | None = None
    """Inputs and their values passed to the test."""

    outputs: List[NamedValue] | None = None
    """Outputs and their values logged by the test."""

    keywords: List[str] | None = None
    """Words or phrases associated with the step."""

    properties: Dict[str, str] | None = None
    """Test step properties."""


class CreateStepRequest(BaseStepRequest):
    name: str
    """Step name."""

    children: List["CreateStepRequest"] | None = None
    """Nested child steps."""



from typing import List

from nisystemlink.clients.core._uplink._with_paging import WithPaging

from ._asset_utilization_history_item import AssetUtilizationHistoryItem


class AssetUtilizationHistoryResponse(WithPaging):
    """Response model for asset utilization history query."""

    asset_utilizations: List[AssetUtilizationHistoryItem] | None = None
    """Array of asset utilizations."""



from datetime import datetime
from enum import Enum, IntEnum
from typing import Any, Dict, List

from nisystemlink.clients.core._uplink._json_model import JsonModel


class AlarmSeverityLevel(IntEnum):
    """Well-known alarm severity levels.

    The service supports custom severity levels greater than 4, but it is generally discouraged.
    The SystemLink Alarm UI only has display strings for severity levels in the range [1, 4].
    """

    CLEAR = -1
    """Severity level for cleared alarms."""

    LOW = 1
    """Low severity level."""

    MODERATE = 2
    """Moderate severity level."""

    HIGH = 3
    """High severity level."""

    CRITICAL = 4
    """Critical severity level."""


class AlarmTransitionType(str, Enum):
    """Specifies a type of alarm transition."""

    CLEAR = "CLEAR"
    """The transition corresponds to the clearing of the condition tracked by the alarm."""

    SET = "SET"
    """The transition corresponds to the condition tracked by the alarm entering into the triggered state."""


class AlarmTransition(JsonModel):
    """A transition within an instance, or occurrence, of an alarm.

    Alarm transitions record changes to an alarm's clear field as well as an alarm
    increasing or decreasing in severity.
    """

    transition_type: AlarmTransitionType
    """Specifies a type of alarm transition: CLEAR or SET."""

    occurred_at: datetime
    """The date and time when the transition occurred."""

    severity_level: int
    """The severity of the transition.

    Valid values for CLEAR transitions are [-1, -1].
    Valid values for SET transitions are [1, 2147483647].
    Note that the SystemLink Alarm UI only has display strings for SET severities in the range [1, 4].
    The AlarmSeverityLevel enum provides values for standard severity levels.
    """

    value: str
    """The value that caused the alarm to transition."""

    condition: str
    """A description of the condition associated with the transition."""

    short_text: str
    """A short description of the condition."""

    detail_text: str
    """A detailed description of the condition."""

    keywords: List[str]
    """Words or phrases associated with a transition.

    Useful for attaching metadata to a transition which could aid in an investigation
    into an alarm's root cause in the future.
    """

    properties: Dict[str, str]
    """Key-value pair metadata associated with a transition.

    Useful for attaching additional metadata to a transition which could aid in an investigation
    into an alarm's root cause in the future. Property keys must be between 1 and 255 characters.
    """


class Alarm(JsonModel):
    """An individual instance, or occurrence, of an alarm.

    The lifecycle of an alarm begins the first time it is triggered and ends when it has been
    both acknowledged and cleared. The service enforces the invariant that there can be at most
    one active=True alarm with the same alarmId.
    """

    instance_id: str
    """The unique identifier for the instance, or occurrence, of the alarm."""

    alarm_id: str
    """A value meant to uniquely identify the particular process or condition tracked by the alarm.

    For example, alarms created by a tag rule engine might have their alarmIds set to the
    concatenation of the path of the tag which caused the rule to be triggered and the ID of the rule.
    """

    workspace: str
    """The workspace the alarm belongs to."""

    active: bool
    """Whether or not the alarm is active.

    An active alarm deserves human or automated attention. Alarms always begin life with active=True.
    This field will be automatically set to false when the clear and acknowledged fields are set to true.
    """

    clear: bool
    """When set to true, the condition that triggered the alarm no longer matches the trigger condition.

    When an alarm is created, clear is initially set to false. The creator of the alarm (typically a
    rule engine or application element of some sort) may determine that the trigger condition no longer
    applies, and may send an update, clearing the alarm. Clearing the alarm does not deactivate the alarm,
    unless the alarm had previously been acknowledged. As long as the alarm is active=true, the alarm may
    transition from clear=True to clear=False (or vice versa) multiple times.
    """

    acknowledged: bool
    """When set to true, the alarm has been acknowledged by an alarm handler, which is typically a human.

    Alarms always begin life with acknowledged=False. Acknowledging an alarm will not affect the active
    flag unless clear is also true. When clear and acknowledged are true, the alarm will become inactive
    (active=False). This field is automatically reset to false when the alarm's highestSeverityLevel field increases.
    """

    acknowledged_at: datetime | None
    """The date and time when the alarm instance was acknowledged.

    This field will be cleared when the alarm's highestSeverityLevel field increases.
    """

    acknowledged_by: str | None
    """The userId of the individual who acknowledged the alarm.

    This field will be cleared when the alarm's highestSeverityLevel field increases.
    """

    occurred_at: datetime
    """The date and time when the alarm was created/first occurred."""

    updated_at: datetime
    """The date and time when the alarm was last updated or modified."""

    created_by: str
    """An identifier for who or what created the alarm.

    This is usually used to identify the particular rule engine which requested that the alarm be created.
    """

    transitions: List[AlarmTransition]
    """A collection of AlarmTransitions for the alarm.

    The service stores the first transition and the most recent 99 other transitions by default.
    The configuration for the service determines the total number of stored transitions per alarm.
    """

    transition_overflow_count: int
    """The number of transitions which overflowed the transitions field for the alarm.

    For example, if the alarm transitioned 250 times, transitionOverflowCount is set to 150 and
    transitions contains the first and most recent 99 transitions. The configuration for the service
    determines the total number of stored transitions per alarm.
    """

    notification_strategy_ids: List[str]
    """The IDs of the notification strategies which will be triggered.

    These are triggered when the alarm is first created and when its highestSeverityLevel increases.
    """

    current_severity_level: int
    """The current severity level of the alarm."""

    highest_severity_level: int
    """The highest severity level that the alarm has ever been in."""

    most_recent_set_occurred_at: datetime | None
    """The date and time of the most recent occurrence of a SET transition.

    This property only considers transitions that cause an alarm state change.
    """

    most_recent_transition_occurred_at: datetime | None
    """The date and time of the most recent occurrence of a transition.

    This property only considers transitions that cause an alarm state change.
    """

    channel: str
    """An identifier for the tag or resource associated with the alarm."""

    condition: str
    """A description of the condition associated with the alarm."""

    display_name: str
    """Optional display name for the alarm. Display names do not need to be unique."""

    description: str
    """Optional description text for the alarm."""

    keywords: List[str]
    """Words or phrases associated with the alarm.

    Alarms can be tagged with keywords to make it easier to find them with queries.
    """

    notes: List[Any]
    """A collection of notes for a given alarm instance.

    Notes are not currently supported and this will always be an empty list.
    """

    properties: Dict[str, str]
    """The alarm's custom properties."""

    resource_type: str
    """The type of resource associated with the alarm."""



from datetime import datetime
from enum import Enum

from nisystemlink.clients.core._uplink._with_paging import WithPaging


class UtilizationOrderBy(Enum):
    """Field by which asset utilization history records can be ordered/sorted."""

    START_TIMESTAMP = "START_TIMESTAMP"


class QueryAssetUtilizationHistoryRequest(WithPaging):
    """Model for object containing filters for asset utilization and assets."""

    utilization_filter: str | None = None
    """
    The filter criteria for asset utilization. Consists of a string of queries
    composed using AND/OR operators. Valid properties: MinionId, Category, UserName, TaskName,
    StartTimestamp, EndTimestamp.
    """

    asset_filter: str | None = None
    """
    The filter criteria for assets. Consists of a string of queries composed
    using AND/OR operators. Valid properties include AssetIdentifier, SerialNumber, ModelName,
    VendorName, Location.MinionId, and many others.
    """

    take: int | None = None
    """The maximum number of asset utilization history records to return."""

    order_by: UtilizationOrderBy | None = None
    """
    The field to order results by. If not provided, default ordering is applied.
    """

    order_by_descending: bool | None = None
    """Whether to return the asset utilization history records in descending order."""

    start_time: datetime | None = None
    """Start of the date range. Defaults to 90 days ago."""

    end_time: datetime | None = None
    """End of the date range. Defaults to current time."""



"""Model for asset identification."""

from nisystemlink.clients.core._uplink._json_model import JsonModel

from ._asset_types import AssetBusType


class AssetIdentification(JsonModel):
    """Model for object containing properties which identify an asset.

    An asset is uniquely identified by a combination of:

    * ``bus_type``
    * ``model_name`` or ``model_number``
    * ``vendor_name`` or ``vendor_number``
    * ``serial_number``
    """

    model_name: str | None = None
    """Model name of the asset."""

    model_number: int | None = None
    """Model number of the asset."""

    serial_number: str | None = None
    """Serial number of the asset."""

    vendor_name: str | None = None
    """Vendor name of the asset."""

    vendor_number: int | None = None
    """Vendor number of the asset."""

    bus_type: AssetBusType | None = None
    """Bus type for the asset."""



from datetime import datetime
from enum import Enum
from typing import List

from nisystemlink.clients.core._uplink._json_model import JsonModel


class TransitionInclusionOption(str, Enum):
    """Determines which transitions to include in the query results."""

    NONE = "NONE"
    """No transitions are included."""

    MOST_RECENT_ONLY = "MOST_RECENT_ONLY"
    """The most recent transition is included."""

    ALL = "ALL"
    """All transitions are included."""


class AlarmOrderBy(str, Enum):
    """The sort order of the returned list of alarms."""

    UPDATED_AT = "UPDATED_AT"
    """The date and time the alarm was last updated."""


class QueryAlarmsWithFilterRequest(JsonModel):
    """Contains filter information for querying alarms."""

    filter: str | None = None
    """The alarm query filter in `Dynamic LINQ`_ format.

    .. _Dynamic LINQ: https://github.com/ni/systemlink-OpenAPI-documents/wiki/Dynamic-Linq-Query-Language

    Allowed properties in the filter are:

    * ``acknowledged``: Boolean value that indicates an alarm acknowledgment
    * ``acknowledgedAt``: DateTime value that represents when the alarm was acknowledged
    * ``active``: Boolean value that indicates an active alarm
    * ``alarmId``: String value that identifies the process or condition tracked by the alarm
    * ``channel``: String value that identifies the tag or resource associated with the alarm
    * ``clear``: Boolean value that indicates an alarm clearance
    * ``createdBy``: String value that identifies the user that created the alarm
    * ``currentSeverityLevel``: Int32 value that represents the severity level of the alarm
    * ``description``: String value that describes the alarm
    * ``displayName``: String value that represents the alarm name in the user interface
    * ``highestSeverityLevel``: Int32 value that represents the highest severity level of the alarm
    * ``keywords``: List of string values associated with the alarm
    * ``mostRecentSetOccurredAt``: DateTime value that represents when the most recent set transition occurred
    * ``mostRecentTransitionOccurredAt``: DateTime value that represents when the most recent transition occurred
    * ``occurredAt``: DateTime value that represents when the alarm was created or first occurred
    * ``occurredWithin``: TimeSpan value that represents when the alarm was created or first occurred
    * ``properties``: Dictionary that contains the string keys and values representing alarm metadata
    * ``resourceType``: String value that represents the resource type of the alarm
    * ``workspace``: String ID of the workspace to use in the query
    * ``workspaceName``: String name of the workspace of the alarm

    Allowed constants in the filter are:

    * ``RelativeTime.CurrentDay``: TimeSpan representing the elapsed time between now and
      the start of the current day
    * ``RelativeTime.CurrentWeek``: TimeSpan representing the elapsed time between now and
      the start of the current week
    * ``RelativeTime.CurrentMonth``: TimeSpan representing the elapsed time between now and
      the start of the current month
    * ``RelativeTime.CurrentYear``: TimeSpan representing the elapsed time between now and
      the start of the current year
    """

    substitutions: List[bool | int | str | None] | None = None
    """Makes substitutions in the query filter expression.

    Substitutions for the query expression are indicated by non-negative integers that are
    prefixed with the @ symbol. Each substitution in the given expression will be replaced by
    the element at the corresponding index (zero-based) in this list.
    """

    return_most_recently_occurred_only: bool = False
    """Whether or not the Alarm Service should group the alarms matched by this query by alarmId.

    When true, only return the most recent alarm for each grouping. In this context, recency is
    based on when the alarm occurred, not when it was last updated.
    """

    transition_inclusion_option: TransitionInclusionOption | None = None
    """Determines which transitions to include in the query results, if any."""

    reference_time: datetime | None = None
    """The date and time to use as the reference point for RelativeTime filters.

    Includes time zone information. Defaults to the current time in UTC.
    """

    take: int | None = 1000
    """Limits the returned list to the specified number of results. Maximum is 1000."""

    order_by: AlarmOrderBy | None = None
    """The sort order of the returned list of alarms.

    By default, alarms are sorted in ascending order based on the specified value.
    """

    order_by_descending: bool = False
    """Whether to sort descending instead of ascending.

    The elements in the list are sorted ascending by default. If true, sorts descending.
    """

    continuation_token: str | None = None
    """A token which allows the user to resume a query at the next item in the matching results.

    When querying, a token will be returned if a query may be continued. To obtain the next page
    of results, pass the token to the service on a subsequent request.
    """

    return_count: bool = False
    """Whether to return the total number of alarms that match the provided filter.

    This disregards the take value.
    """



"""Model for updating utilization (heartbeat or end)."""

from datetime import datetime
from typing import List

from nisystemlink.clients.core._uplink._json_model import JsonModel


class _UpdateUtilizationRequest(JsonModel):
    """Request model for updating utilization (heartbeat or end)."""

    utilization_identifiers: List[str] | None = None
    """Array representing the unique identifier of an asset utilization history record."""

    utilization_timestamp: datetime | None = None
    """A date time value which can be used to specify the heartbeat timestamp."""



from typing import List

from nisystemlink.clients.alarm.models._alarm_instances_partial_success import (
    AlarmInstancesPartialSuccess,
)


class AcknowledgeAlarmsResponse(AlarmInstancesPartialSuccess):
    """Contains information about which alarms were acknowledged."""

    acknowledged: List[str]
    """The instanceIds of the alarms that were successfully acknowledged."""



"""Model for start utilization partial success response."""

from typing import List

from nisystemlink.clients.core import ApiError
from nisystemlink.clients.core._uplink._json_model import JsonModel

from ._asset_identification import AssetIdentification


class StartUtilizationPartialSuccessResponse(JsonModel):
    """Response model for start utilization operation with partial success support."""

    error: ApiError | None = None
    """Error information if any failures occurred."""

    assets_with_started_utilization: List[AssetIdentification] | None = None
    """Array containing the asset identification data for the assets that started being utilized."""

    failed: List[AssetIdentification] | None = None
    """Array containing the asset identification data for the assets that failed to start being utilized."""



from typing import List

from nisystemlink.clients.core._uplink._with_paging import WithPaging

from ._alarm import Alarm


class QueryAlarmsWithFilterResponse(WithPaging):
    """Contains information about the alarms matched by a query."""

    alarms: List[Alarm]
    """The list of alarms returned by the query."""

    total_count: int | None = None
    """The total number of alarms which matched the query, if requested."""



from typing import Dict

from nisystemlink.clients.core._uplink._json_model import JsonModel


class Dashboard(JsonModel):
    """Represents a dashboard reference."""

    id: str | None = None
    """ID of the dashboard"""

    variables: Dict[str, str] | None = None
    """Variables for the dashboard"""


class DashboardUrl(Dashboard):
    """Definition and URL of the dashboard reference associated with this test plan."""

    url: str | None = None
    """URL of the dashboard reference associated with this test plan."""



from typing import List

from nisystemlink.clients.alarm.models._alarm_instances_partial_success import (
    AlarmInstancesPartialSuccess,
)


class DeleteAlarmsResponse(AlarmInstancesPartialSuccess):
    """Contains information about alarms that were deleted."""

    deleted: List[str]
    """The instanceIds of the alarms that were successfully deleted."""



# -*- coding: utf-8 -*-

"""Helper function to get minion ID from Salt configuration."""

from nisystemlink.clients.core._internal._path_constants import PathConstants


def read_minion_id() -> str | None:
    """Read the minion ID from the Salt configuration.

    Returns:
        str | None: The minion ID content if the file exists, None otherwise.
    """
    minion_id_path = PathConstants.salt_data_directory / "conf" / "minion_id"

    if not minion_id_path.exists():
        return None

    try:
        with open(minion_id_path, "r", encoding="utf-8") as fp:
            return fp.read().strip()
    except (OSError, PermissionError):
        return None



"""Model for update utilization partial success response."""

from typing import List

from nisystemlink.clients.core import ApiError
from nisystemlink.clients.core._uplink._json_model import JsonModel


class UpdateUtilizationPartialSuccessResponse(JsonModel):
    """Response model for update utilization operation with partial success support."""

    error: ApiError | None = None
    """Error information if any failures occurred."""

    updated_utilization_ids: List[str] | None = None
    """Array of utilization identifiers for the entries that were updated."""

    failed: List[str] | None = None
    """Array of utilization identifiers for the entries that failed to update."""



from typing import Dict, List

from nisystemlink.clients.core._uplink._json_model import JsonModel

from ._dashboard import Dashboard
from ._execution_definition import ExecutionDefinition


class CreateTestPlanRequest(JsonModel):
    """Represents the request body content for creating a test plan."""

    name: str | None = None
    """The name of the test plan."""

    state: str | None = None
    """The state of the test plan."""

    template_id: str | None = None
    """The ID of the template to use for the test plan."""

    description: str | None = None
    """A description of the test plan."""

    assigned_to: str | None = None
    """The user or group assigned to the test plan."""

    part_number: str | None = None
    """The part number associated with the test plan."""

    dut_id: str | None = None
    """The Device Under Test (DUT) ID."""

    dut_serial_number: str | None = None
    """The Device Under Test (DUT) serial number."""

    test_program: str | None = None
    """The test program associated with the test plan."""

    work_order_id: str | None = None
    """The work order ID associated with the test plan."""

    estimated_duration_in_seconds: int | None = None
    """The estimated duration of the test plan in seconds."""

    system_filter: str | None = None
    """The system filter to apply."""

    dut_filter: str | None = None
    """The DUT filter to apply."""

    execution_actions: List[ExecutionDefinition] | None = None
    """List of execution actions for the test plan."""

    file_ids_from_template: List[str] | None = None
    """List of file IDs from the template."""

    workspace: str | None = None
    """The workspace associated with the test plan."""

    properties: Dict[str, str] | None = None
    """Additional properties for the test plan."""

    dashboard: Dashboard | None = None
    """The dashboard associated with the test plan."""
